--------------------------------------------------------------------------------
-- House of Winters - UI controls library.
--------------------------------------------------------------------------------

local MAJOR, MINOR = "LibHoWUI-1.0", 0
local howui, oldminor = LibStub:NewLibrary(MAJOR, MINOR)
if not howui then return end	--the same or newer version of this lib is already loaded into memory

--- Create a top level window.
-- @param name unique window name.
-- @param dimension the window dimensions table {width: 500, height: 500}.
-- @param anchor the anchor config [default: TOPLEFT,GuiRoot,nil,0,0].
-- @param clampedToScreen set clampedToScreen state.
-- @param movable set window movable state.
-- @param mouseEnabled set window mouseEnabled state.
-- @param hidden set window hidden state.
-- @return the created top level control.
function howui:TopLevelWindow(name, dimension, anchor, clampedToScreen, movable, mouseEnabled, hidden)
    -- Required stuff without default values.
    local window = WINDOW_MANAGER:CreateTopLevelWindow(name)
    window:SetDrawLevel(0)

    if dimension then
        window:SetDimensions(dimension.width, dimension.height)
    end

    if anchor then
        window:SetAnchor(anchor.point, anchor.relativeTo, anchor.relativePoint, anchor.offsetX, anchor.offsetY)
    end

    if clampedToScreen then
        window:SetClampedToScreen(true)
    end

    if movable then
        window:SetMovable(true)
    end

    if mouseEnabled then
        window:SetMouseEnabled(true)
    end

    if hidden then
        window:SetHidden(true)
    end

    return window
end

--- Create a Button control.
-- @param name unique control name.
-- @param text the button text string.
-- @param dimension the button dimensions table {width: 100, height: 30}.
-- @param parent the parent control [default: GuiRoot].
-- @param anchor the anchor config [default: TOPLEFT,GuiRoot,nil,0,0].
-- @param font the font type [default: ZoGameFont].
-- @param scale the scale [default: 1].
-- @param hidden set button hidden state.
-- @return the created button control.
function howui:Button(name, text, dimension, parent, anchor, font, scale, hidden)
    -- Required stuff without default values.
    if not parent then
        parent = GuiRoot
    end
    local control = WINDOW_MANAGER:CreateControl(name, parent, CT_BUTTON)
    if text then
        control:SetText(text)
    end
    if dimension.width and dimension.height then
        control:SetDimensions(dimension.width, dimension.height)
    end
    control:SetHorizontalAlignment(CENTER)
    control:SetVerticalAlignment(CENTER)

    -- Optional stuff with default values.
    if not anchor then
        anchor = {
            point = TOPLEFT,
            relativeTo = GuiRoot,
            relativePoint = nil,
            offsetX = 0,
            offsetY = 0
        }
    end
    control:SetAnchor(anchor.point, anchor.relativeTo, anchor.relativePoint, anchor.offsetX, anchor.offsetY)

    if not font then
        font = "ZoGameFont"
    end
    control:SetFont(font)

    if not scale then
        scale = 1
    end
    control:SetScale(scale)

    if hidden then
        control:SetHidden(true)
    end

    return control
end

--- Create a Texture control.
-- @param name unique control name.
-- @param filename the texture filename.
-- @param parent the parent control [default: GuiRoot].
-- @param hidden set label hidden state.
-- @return the created label control.
function howui:Texture(name, filename, parent, hidden)
    -- Required stuff without default values.
    if not parent then
        parent = GuiRoot
    end
    local control = WINDOW_MANAGER:CreateControl(name, parent, CT_TEXTURE)
    if filename then
        control:SetTexture(filename)
    end
    control:SetAnchorFill()

    if hidden then
        control:SetHidden(true)
    end

    return control
end

--- Create a label control.
-- @param name unique control name.
-- @param text the label text string.
-- @param dimension the label dimensions table {width: 100, height: 30}.
-- @param parent the parent control [default: GuiRoot].
-- @param anchor the anchor config [default: TOPLEFT,GuiRoot,nil,0,0].
-- @param font the font type [default: ZoGameFont].
-- @param color the text color [default: 1,1,1,1].
-- @param hidden set label hidden state.
-- @return the created label control.
function howui:Label(name, text, dimension, parent, anchor, font, color, hidden)
    -- Required stuff without default values.
    if not parent then
        parent = GuiRoot
    end
    local control = WINDOW_MANAGER:CreateControl(name, parent, CT_LABEL)
    if text then
        control:SetText(text)
    end
    if dimension.width and dimension.height then
        control:SetDimensions(dimension.width, dimension.height)
    end

    -- Optional stuff with default values.
    if not anchor then
        anchor = {
            point = TOPLEFT,
            relativeTo = GuiRoot,
            relativePoint = nil,
            offsetX = 0,
            offsetY = 0
        }
    end
    control:SetAnchor(anchor.point, anchor.relativeTo, anchor.relativePoint, anchor.offsetX, anchor.offsetY)

    if not font then
        font = "ZoGameFont"
    end
    control:SetFont(font)

    if not color then
        color = {
            red = 1,
            green = 1,
            blue = 1,
            alpha = 1
        }
    end
    control:SetColor(color.red, color.green, color.blue, color.alpha)

    if hidden then
        control:SetHidden(true)
    end

    return control
end

--- Append text to a label control.
-- @param control the control to append text to.
-- @param text the text string to append
-- @return void
function howui:AppendText(control, text)
    local original = control:GetText()
    if original and original ~= "" then
        control:SetText(original.."\n"..text)
    else
        control:SetText(text)
    end
end
